#ifndef __CDCTRACKMAKER__
#define __CDCTRACKMAKER__

//*************************************************************************
//* -------------------
//* CDCTrackMaker Class
//* -------------------
//* (Description)
//*	This set of classes makes tracks.
//* (Requires)
//*	TObjArray:
//*	JSFSteer:
//*	JSFModules:
//*	CDCHit:
//*	CDCHitBuf:
//*	CDCHitMaker:
//* (Provides)
//*	CDCTrackMaker		: steering class
//*	CDCTrackBuf		: track data buffer class
//*	CDCTrack		: track data class
//*	CDCTrackHit		: track hit
//* (Update Record)
//*	2000/01/07  K.Fujii	Original version.
//*	2000/09/30  K.Fujii	Added xDXpivot and xKCNsigmas data members
//*				to CDCTrackBuf, and then their setters:
//*				SetDXpivotCut and SetKCNsigCut.
//*************************************************************************

#include "CDCHitMaker.h"
#include "Lockable.h"

// ---------
// Track Hit
// ---------

class CDCTrackHit : public TObject, public Lockable {
friend class CDCTrack;
public:
  CDCTrackHit();
  CDCTrackHit(CDCHit *ht, Int_t lr);
  CDCTrackHit(CDCTrackHit &hit);
  virtual ~CDCTrackHit();

  inline Double_t 	GetResidual () const    { return fResidual; }
  inline Int_t		GetLeftRight() const    { return fLeftRight; }
  inline CDCHit*	GetCDCHit   () const	{ return fHit; }
  inline void		SetResidual(Double_t r) { fResidual = r; }

private:
  Double_t fResidual;		// residual = hit - track (cm)
  Int_t	   fLeftRight;		// (0,1) = (left,right)
  CDCHit  *fHit;		// pointer to CDC hit

  ClassDef(CDCTrackHit, 1) 	// CDC Track Hit
};

// -----------
// Track Datum
// -----------

class CDCTrack : public TObjArray, public Lockable {
public:
  CDCTrack();
  CDCTrack(CDCTrack &t);
  virtual ~CDCTrack();

  Bool_t	IsSortable() const;
  Int_t		Compare(TObject *obj);
  void 		Add(TObject *obj);
  void		SetTrackType(Int_t type);
  Int_t		GetTrackType();
  Double_t	GetChi2();
  Double_t      Xat(Double_t y);
  Double_t      SGX2at(Double_t y);

  void 		DebugPrint(const Char_t *opt = "Brief");
  void		Update();

private:
  void 		Fit();			// fit and return chi^2

private:
  Bool_t	fDone;			// (1,0) = (fit done, not yet)
  Int_t		fTrackType;		// track type
  Double_t	fChi2;			// chi^2
  Double_t	fa;			// a when x = a*y + b
  Double_t	fb;			// b
  Double_t      fEaa;			// <da*da>
  Double_t	fEba;			// <db*da>
  Double_t	fEbb;			// <db*db>

  ClassDef(CDCTrack, 1) 	// CDC Track
};

// ------------
// Track Buffer
// ------------
class CDCTrackMaker;

class CDCTrackBuf : public JSFEventBuf {
friend class CDCTrackMaker;
public:
  CDCTrackBuf(const char *name="CDCTrackBuf",
	      const char *title="CDC Track data", 
	      CDCTrackMaker *mod=0);
  CDCTrackBuf(CDCTrackMaker *mod,
	       const char *name="CDCTrackBuf",
	       const char *title="CDC Track data");
  virtual ~CDCTrackBuf();

  inline void	SetChi2Cut(Double_t x) { xChi2    = x; }
  inline void 	SetNsigCut(Double_t x) { xNsigmas = x; }
  inline void 	SetDXpivotCut(Double_t x) { xDXpivot   = x; }
  inline void 	SetKCNsigCut(Double_t x)  { xKCNsigmas = x; }
  void		SetHitData(CDCHitBuf *htbuf);

  Bool_t	IsInitialized() const;	// inquires if input data set
  Int_t		GetNtracks();		// returns No. tracks
  TObjArray    *GetTracks();		// returns array of tracks

  void 		DebugPrint(const Char_t *opt = "Brief");

private:
  void		MakeVectorHits(CDCHitCell *cell); // make vector hits
  void		MakeTracks();			  // make tracks

private:
  Bool_t	      fDone;	 	//  (1,0) = (tracking done, not yet)
  CDCHitBuf	     *fHitData;		//  array of layers
  TObjArray	      fTracks; 		//  array of tracks
  Double_t    	      xNsigmas;		//  window for residual
  Double_t 	      xChi2;		//  chi^2 cut on a vector hit
  Double_t	      xDXpivot;		//  dx cut for pivoits
  Double_t	      xKCNsigmas;	//  window for hit poisoning

  ClassDef(CDCTrackBuf, 1) // CDC track data buffer
};


// -----------
// Track Maker
// -----------

class CDCTrackMaker : public JSFModule {
public:
  CDCTrackMaker(const Char_t *name="CDCTrackMaker",
		const Char_t *title="CDC Track Maker");
  virtual ~CDCTrackMaker();

  Bool_t 	Initialize();		// job initialization
  Bool_t 	BeginRun(Int_t run);	// process Begin Run record
  Bool_t 	Process(Int_t ev);	// process event
  
  ClassDef(CDCTrackMaker, 1) 	// CDC Track Maker
};

#endif







